@@ -9,7 +9,7 @@ use warnings;
 use strict;
 use Carp;
 
-our $VERSION = '0.20_05';
+our $VERSION = 0.01;
 
 
 sub error
@@ -52,16 +52,12 @@ sub write_info_error
     }
 }
 
-# Return the verbosity.
-
 sub verbose
 {
     my ($png) = @_;
     return $png->{verbosity};
 }
 
-# Set the verbosity.
-
 sub verbosity
 {
     my ($png, $verbosity) = @_;
@@ -71,13 +67,12 @@ sub verbosity
     $png->{verbosity} = 1;
 }
 
-# Make the object.
-
 sub new
 {
     my ($package, $options) = @_;
     my $png = {};
     bless $png;
+    $png->{read} = undef;
     # The marker "error_string" contains the most recent error.
     $png->{error_string} = '';
     if ($options && ref $options eq 'HASH') {
@@ -91,8 +86,6 @@ sub new
     return $png;
 }
 
-# Read a file
-
 sub Image::PNG::read
 {
     my ($png, $file_name) = @_;
@@ -100,7 +93,7 @@ sub Image::PNG::read
         print "I am going to try to read a file called '$file_name'.\n";
     }
     if (! defined $file_name) {
-        carp __PACKAGE__, ": You called 'read' without giving a file name";
+        carp __PACKAGE__, ": You called 'read_file' without giving a file name";
         return;
     }
     my $read = Image::PNG::Container->new ({read_only => 1});
@@ -172,7 +165,7 @@ sub Image::PNG::write
             if ($png->verbose) {
                 print "I am setting the image data for the image to write to data which I read in from another image.";
             }
-            my $rows = Image::PNG::Libpng::get_rows ($png->{read}->png ());
+            my $rows = Image::PNG::Libpng::get_rows ($png->{read_png});
             if ($png->verbose) {
                 print "I've got the data from the read image and now I'm going to set up the writing to write that data.\n";
             }
@@ -184,9 +177,8 @@ sub Image::PNG::write
             return;
         }
     }
-    if ($png->verbose) {
-        printf ("Its colour type is %s.\n", Image::PNG::Libpng::color_type_name ($write->{ihdr}->{color_type}));
-    }
+    printf ("Its colour type is %s.\n", Image::PNG::Libpng::color_type_name ($write->{ihdr}->{color_type}));
+
     if ($write->{ihdr}->{color_type} == PNG_COLOR_TYPE_PALETTE) {
         if ($png->verbose) {
             print "The image you want to write has a palette, so I am going to check whether the palette is ready to be written.\n";
@@ -195,7 +187,7 @@ sub Image::PNG::write
             print "The image doesn't have a palette set.\n";
             if ($png->{read}) {
                 print "I am going to try to get one from the image I read in.\n";
-                my $palette = Image::PNG::Libpng::get_PLTE ($png->{read}->png ());
+                my $palette = Image::PNG::Libpng::get_PLTE ($png->{read_png});
                 for my $color (@$palette) {
                     for my $hue (qw/red green blue/) {
                         printf "%s: %d ", $hue, $color->{$hue};
@@ -253,8 +245,8 @@ sub write_set
     $png->{write}->{$key} = $value;
 }
 
-# Initialize the object $png for writing (get the libpng things we
-# need to write an image, set a flag "write_ok" in the image.).
+# Initialize the object $png for writing (get the stupid libpng things
+# we need to write an image, set a flag "write_ok" in the image.).
 
 sub init_write
 {
@@ -284,13 +276,11 @@ sub data
     my ($png, $data) = @_;
     if (! $data) {
         # Return the existing data
-    }
-    else {
+    } else {
         # Put $data into the PNG
         if ($png->{data}) {
             carp __PACKAGE__, ": you have asked me to put a scalar value as the PNG data, but I already have PNG data inside me. Please use a fresh object.";
-        }
-        elsif ($png->{read_file_name}) {
+        } elsif ($png->{read_file_name}) {
             carp __PACKAGE__, ": you have asked me to put a scalar value as the PNG data, but I already contain PNG data from a file called '$png->{read_file_name}. Please use a fresh object.";
         }
     }
@@ -308,7 +298,7 @@ sub IHDR
         $png->{write}->{ihdr_set} = 1;
     }
     else {
-        $ihdr = Image::PNG::Libpng::get_IHDR ($png->{read}->png ());
+        $ihdr = Image::PNG::Libpng::get_IHDR ($png->{read_png});
     }
     return $ihdr;
 }
@@ -325,8 +315,6 @@ sub get_IHDR
     $png->{IHDR} = Image::PNG::Libpng::get_IHDR ($png->{read}->png ());
 }
 
-# Get $key from the PNG.
-
 sub get
 {
     my ($png, $key) = @_;
@@ -336,8 +324,6 @@ sub get
     return $png->{IHDR}->{$key};
 }
 
-# Get/set width of PNG
-
 sub width
 {
     my ($png, $width) = @_;
@@ -349,8 +335,6 @@ sub width
     }
 }
 
-# Get/set height of PNG
-
 sub height
 {
     my ($png, $height) = @_;
@@ -409,7 +393,7 @@ sub rows
             #        $png->handle_error ("");
             return;
         }
-        return Image::PNG::Libpng::get_rows ($png->{read}->png ());
+        return Image::PNG::Libpng::get_rows ($png->{read_png});
     }
 }
 
@@ -420,7 +404,7 @@ sub rowbytes
 #        $png->handle_error ("");
         return;
     }
-    return Image::PNG::Libpng::get_rowbytes ($png->{read}->png ());
+    return Image::PNG::Libpng::get_rowbytes ($png->{read_png});
 }
 
 sub text
@@ -428,7 +412,7 @@ sub text
     my ($png, $text) = @_;
     if (! $png->{text}) {
         my $text_ref =
-            Image::PNG::Libpng::get_text ($png->{read}->png ());
+            Image::PNG::Libpng::get_text ($png->{read_png});
         $png->{text} = $text_ref;
         # Change the text compression field to words rather than numbers.
         for my $text (@{$png->{text}}) {
@@ -445,10 +429,18 @@ sub text
 sub time
 {
     my ($png) = @_;
-    if (! $png->{read}) {
-        return undef;
+    return Image::PNG::Libpng::get_tIME ($png->{read_png});
+}
+
+# We need to free the memory associated with the C structs allocated
+# inside libpng, so this class has a DESTROY method.
+
+sub DESTROY
+{
+    my ($png) = @_;
+    if ($png->{verbose}) {
+        print "Freeing the memory of a PNG object.\n";
     }
-    return Image::PNG::Libpng::get_tIME ($png->{read}->{png});
 }
 
 # The text segment of the PNG should probably be an object in its own
@@ -486,9 +478,22 @@ sub display_text
 
 1;
 
+
 =head1 NAME
 
-Image::PNG - Read and write PNG files
+Image::PNG
+
+Read and write PNG (Portable Network Graphics) files
+
+=head1 WARNING
+
+This version of the module is solely for evaluation and testing. This
+module is currently incomplete and untested, and contains errors,
+bugs, and inconsistencies, including unresolved memory corruption
+errors which can crash Perl. The documentation below refers to
+functions which may or may not exist in the completed module and
+contains links to things which do not exist and may never exist.
+
 
 
 
@@ -498,66 +503,23 @@ Image::PNG - Read and write PNG files
     $png->read_file ("crazy.png");
     printf "Your PNG is %d x %d\n", $png->width, $png->height;
 
-=head1 DESCRIPTION
-
-Image::PNG is a layer on top of L<Image::PNG::Libpng>. Whereas
-L<Image::PNG::Libpng> copies the interface of the C library C<libpng>,
-Image::PNG is intended to be a more intuitive way to handle PNG
-images. 
-
-Please note that this module is not completed yet and is still under
-development, so the interface may change. It's also open to
-suggestions for improvements.
-
 =head1 General methods
 
-=head2 new
+=head2 read_file
 
-    my $png = Image::PNG->new ();
-
-Create a new PNG-file reading or writing object.
-
-Options are
-
-=over
-
-=item read
-
-    my $png = Image::PNG->new ({read => 'some.png'});
-
-Set the file to read. The file is then read at the time of object
-creation.
-
-=item verbosity
-
-    my $png = Image::PNG->new ({verbosity => 1});
-
-If C<verbosity> is set to a true value, print verbose messages about
-what the module is doing.
-
-=back
-
-=head2 read
-
-    $png->read ("crazy.png")
+    $png->read_file ("crazy.png")
         or die "Can't read it: " . $png->error ();
 
-Read the PNG from the file name specified as the argument. This dies
-if there is an error.
+=head2 write_file
 
-=head2 write
-
-    $png->write ("crazy.png")
+    $png->write_file ("crazy.png")
         or die "Can't write it: " . $png->error ();
 
-Write the PNG to the file name specified as the argument. This dies
-if there is an error.
-
 =head2 data
 
      my $data = $png->data ();
 
-Get the PNG image data as a Perl scalar.
+Get the PNG binary data.
 
 =head2 error
 
@@ -609,7 +571,7 @@ Get the bit depth of the current PNG image.
 
 =head2 interlacing_method
 
-    my $interlacing_method = $png->interlacing_method ();
+    my $interlacing_method = $png->interlacing_method
 
 Get the name of the method of interlacing of the current PNG image.
 
@@ -629,13 +591,11 @@ no image has been read yet, it returns the undefined value.
 
     my $rows = $png->rows;
 
-This method returns the rows of the image as an array reference,
-C<$rows>. The array reference is a size equal to the height of the
-image. Each element has the length of the number of bytes in one row
-(as given by L</rowbytes>) plus one final zero byte. 
-
-The row data returned is binary data and may contain several bytes
-with the value zero.
+This method returns the rows of the image as an array reference which
+contains a number of elements equal to the height of the image. Each
+element has the length of the number of bytes in one row (as given by
+L<rowbytes>) plus one final zero byte. The row data returned is binary
+data and may contain several bytes with the value zero.
 
 =head1 Non-image chunks
 
@@ -643,10 +603,9 @@ with the value zero.
 
     my @text = $png->text;
 
-Get the text chunks of the image. Each element of the return value is
-a hash reference. The keys are the fields of the PNG text chunk, and
-the values are the values of those fields in the text chunk. The size
-of the array is equal to the number of text chunks.
+Get the text chunks of the image. Each chunk is a hash reference with
+the keys being the fields of the PNG text chunk and the values being
+the values of those fields.
 
 =head2 time
 
@@ -654,7 +613,7 @@ of the array is equal to the number of text chunks.
     print "The PNG was last modified in $time_ref->{year}.\n";
 
 Get the last modified time of the image. The return value is a hash
-reference containing the following six fields,
+reference containing six fields,
 
 =over
 
@@ -674,10 +633,10 @@ reference containing the following six fields,
 
 These represent the last modification time of the image. The
 modification time of a PNG file is meant to be in the GMT (UCT) time
-zone so there is no time zone information.
+zone so there is no time zone information in this.
 
-If there is no last modification time, the undefined value is returned
-instead of a hash reference.
+If there is no last modification time, a hash reference is returned
+but it doesn't contain any fields.
 
 =head1 FUNCTIONS
 
@@ -691,15 +650,16 @@ There are some convenience functions in this module, exported on request.
 
 Display the text chunk given as an argument on C<STDOUT>.
 
-This is a convenience function for debugging rather than a
-general-purpose routine.
+This is meant as a minimal convenience function for when you are
+debugging or something rather than a general-purpose text chunk
+display routine.
 
 =head1 SUPPORT
 
-There is L<a mailing list for this Perl module at Google
-Groups|https://groups.google.com/forum/#!forum/perlimagepng>. If you
-have an enquiry, post to the mailing list. You don't have to join the
-mailing list to post a message.
+There is a mailing list for this Perl module at Google Groups. If you
+have a question or suggestion or bug report, please let me know via
+the mailing list. You don't have to join the mailing list to post a
+message.
 
 =head1 SEE ALSO
 
@@ -717,7 +677,7 @@ PNG library "libpng". Image::PNG is built on top of this module.
 
 =head2 libpng download
 
-To download libpng, see
+If you need to download libpng, see
 L<http://www.libpng.org/pub/png/libpng.html>. See also L</Alien::PNG>.
 
 =head2 Other Perl modules on CPAN
@@ -739,49 +699,10 @@ users whether this works or not on various platforms.
 =head3 Image::PNG::Rewriter
 
 L<Image::PNG::Rewriter> is a utility for unpacking and recompressing
-the IDAT (image data) part of a PNG image. The main purpose seems to
-be to recompress the image data with the module author's other module
-L<Compress::Deflate7>. Unfortunately that only works with Perl
-versions 5.12.
-
-=head3 Image::Pngslimmer
-
-L<Image::Pngslimmer> reduces the size of dynamically created PNG
-images. It's very, very slow at reading PNG data, but seems to work
-OK.
-
-=head3 Image::Info
-
-L<Image::Info> is a module for getting information out of various
-types of images. It has good support for PNG and is written in pure
-Perl (doesn't require a C compiler). As well as basics such as height,
-width, and colour type, it can get text chunks, modification time,
-palette, gamma (gAMA chunk), resolution (pHYs chunk), and significant
-bits (sBIT chunk). At the time of writing (version 1.31) it doesn't
-support other chunks.
-
-=head3 Image::Size
-
-If you only need to find the size of an image, L<Image::Size> can give
-you the size of PNG and various other image formats. This module is
-highly recommended on CPAN ratings.
-
-=head3 Image::PNGwriter
-
-L<Image::PNGwriter> is an interface to a project called
-"PNGwriter". At the time of writing (2013-12-01), only one version has
-been released, in 2005. The most recent version of PNGwriter itself is
-from 2009.
-
-
-
+the IDAT (image data) part of a PNG image.
 
 =head2 About the PNG format
 
-=head3 Wikipedia article
-
-There is L<an article on the format on Wikipedia|http://en.wikipedia.org/wiki/Portable_Network_Graphics>.
-
 =head3 The PNG specification
 
 L<The PNG specification|http://www.w3.org/TR/PNG/> (link to W3
@@ -792,30 +713,11 @@ consortium) explains the details of the PNG format.
 
 The book "PNG - The Definitive Guide" by Greg Roelofs, published in
 1999 by O'Reilly is available online at
-L<http://www.faqs.org/docs/png/>. 
-
-=head1 EXAMPLES
-
-There is a collection of demonstrations and example scripts online at
-L<http://www.lemoda.net/image-png/>. This currently contains
-
-=over
-
-=item PNG inspector L<http://www.lemoda.net/png/inspect/>
-
-This demonstration downloads a PNG file you specify from the internet
-and prints out its contents.
-
-=item PNG quantizer L<http://www.lemoda.net/png/quantize/>
-
-This demonstration downloads a PNG file you specify from the internet
-and quantizes it with as many colours as you want (from two to 256) in
-order to reduce its size. The quantized image is uploaded to imgur.com
-using L<WWW::Imgur>.
-
-=back
-
-
+L<http://www.faqs.org/docs/png/>. I didn't refer to this book at all
+in making Image::PNG, so I can't vouch for it, but looking at the
+contents pages it appears to contain a lot of useful information,
+although it is definitely showing its age, with chapters about
+software such as Netscape Navigator and BeOS.
 
 =head1 AUTHOR
 
@@ -824,7 +726,7 @@ Ben Bullock, <bkb@cpan.org>
 =head1 COPYRIGHT & LICENCE
 
 The Image::PNG package and associated files are copyright (C)
-2014 Ben Bullock.
+2011 Ben Bullock.
 
 You can use, copy, modify and redistribute Image::PNG and
 associated files under the Perl Artistic Licence or the GNU General
@@ -835,24 +737,43 @@ Public Licence.
 =head2 Mailing list
 
 There is a mailing list at
-L<http://groups.google.com/group/perlimagepng>. It is not necessary to
-be a member of the list to post messages to the list or participate in
-discussions.
+L<http://groups.google.com/group/perlimagepng>. You don't need to be a
+member of the list to post messages to the list or participate in
+discussions. Your messages may be held for moderation though.
+
+If you have anything at all to say about the module, whether it is bug
+reports, feature requests, or anything else, I'd prefer to discuss it
+on the mailing list because that way there is a public record which
+everyone can access, rather than being restricted to email. That means
+that, for example, if someone else takes over maintaining this module,
+they can easily access records of previous discussions.
 
 =head2 CPAN stuff
 
-There is a bug tracker at L<https://github.com/benkasminbullock/Image-PNG/issues>.
+There is a bug tracker at .
 
 =head1 FOR PROGRAMMERS
 
-The distributed files are not the source code of the module. The
-source code lives in the "tmpl" directory of the distribution and the
-distribution is created via scripts.
+(This section is only for people who want to fix a bug or add an
+improvement to this module, and who want to share the change with
+other people by adding it to the public version of this module.)
+
+If you want to alter this module, please note very carefully that the
+distributed files are not actually the source code of the module. If
+you wish to obtain the source code of Image::PNG, please access
+http://www.lemoda.net/projects/image-png/repo/ using the "git" source control system.
+
+The original files of this distribution make very heavy use of the
+L<Template> Perl module in order to cut down the amount of repetitive
+stuff which needs to be put into various source and documentation
+files. If you plan to alter the C file, you also need a program called
+"cfunctions" which you can download from sourceforge.
 
 
 
 =cut
 
+
 # Local Variables:
 # mode: perl
 # End:
